package com.android.ptplib;

/* interface to be implemented in BaselinInititor and its subclasses
 * Copyright 2012 flying_galactic
 */

public interface CameraFunctions {
	
	/** Vendor specific functions, must be impelmented and overridden 
	 *  in subclasses of the BaselinInititor !
	 * 
	 */
	
	public Response initiateCapture(int storageId, int formatCode) throws PTPException;
		
	public Response startBulb () throws PTPException;

	public Response stopBulb () throws PTPException;

	public Response setShutterSpeed (int speed) throws PTPException;  

	public Response setExposure(int exposure) throws PTPException;

	public Response setISO(int value) throws PTPException;

	public Response setAperture(int value) throws PTPException;

	public Response setImageQuality(int value) throws PTPException;
	
	public Response getDevicePropValue(int property, DevicePropDesc desc) throws PTPException;
	
	
	/** Floating point adapters, must be overridden in subclasses of the BaselinInititor
	 * 
	 */

	public Response setShutterSpeed (double timeSeconds) throws PTPException;

	public Response setAperture(double apertureValue) throws PTPException;

	// Sets ISO to 50, 100, 200... or nearest value
	public Response setISO(double isoValue) throws PTPException;

	public Response setExposure(double exposureValue) throws PTPException;

	/**  Selects image Quality from "S" to "RAW" at least in 4 steps 
	 *   implement in subclasses of the BaselinInititor
	 */
	public Response setImageQuality (String quality) throws PTPException;

	
	/** Functions which wait until device not busy, implemented in BaselinInititor
	 *  they call the above function
	 * 
	 */
	
	public Response initiateCapture(int storageId, int formatCode, int busyTimeoutMillis) throws PTPException;

	public Response setShutterSpeed (double timeSeconds, int busyTimeoutMillis) throws PTPException;  

	public Response setExposure(double exposureValue, int busyTimeoutMillis) throws PTPException;

	public Response setISO(double isoValue, int busyTimeoutMillis) throws PTPException;

	public Response setAperture(double apertureValue, int busyTimeoutMillis) throws PTPException;

	public Response setImageQuality (String quality, int busyTimeoutMillis) throws PTPException;

	/** Calls BulbStart and BulbEnd, implemented in BaselineInitiator
	 * 
	 * @param milliseconds
	 * @return
	 * @throws PTPException
	 */
	public Response bulbExposure (int milliseconds)  throws PTPException;


}
