package com.android.ptplib;

/* Android Class for managing an USB connection do digital cameras. 
 * (C) 2012 Wolfgang Strickling aka. flying_galactic
 * 
 * 
 * usage: see ReadMe.txt of PtpLib 
 * 
 */


import com.android.ptplib.eos.EosInitiator;     // device specific access class for CANON EOS cameras
import com.android.ptplib.nikon.NikonInitiator; // device specific access class for CANON EOS cameras

import android.annotation.SuppressLint;
import android.app.PendingIntent;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.hardware.usb.UsbDevice;
import android.hardware.usb.UsbDeviceConnection;
import android.hardware.usb.UsbManager;
import android.os.Build;
import android.os.Handler;
import android.os.Message;
import android.util.Log;

@SuppressLint("NewApi")
public class CameraManager {
	
	public  static String 	    ACTION_USB_PERMISSION = "com.android.ptplib.USB_PERMISSION"; // may be modified before use
	private PendingIntent 		mPermissionIntent;
	public  static BaselineInitiator	bi;
	public  String				permissionRequested = "";
	
	public  static String 		TAG = "CameraManager";
	private Context				appContext;
	private Handler 			handler;
	public  static final int	MSG_CONNECTED 		= 100;
	public  static final int	MSG_DISCONNECTED 	= 101;
	
	//constructor
	public CameraManager (Context appContext, Handler handler){	
		this.appContext = appContext;
		this.handler	= handler;
		if (Build.VERSION.SDK_INT > 11) {
			IntentFilter filter = new IntentFilter();		
			filter.addAction(UsbManager.ACTION_USB_DEVICE_ATTACHED);
			filter.addAction(UsbManager.ACTION_USB_DEVICE_DETACHED);
			filter.addAction(ACTION_USB_PERMISSION);
			appContext.registerReceiver(mUsbReceiver, filter);	
			mPermissionIntent = PendingIntent.getBroadcast(appContext, 0, new Intent(ACTION_USB_PERMISSION), 0);
		}
	}
	
	/**
	 *   call requestPermission (); // in onResume
	 */
	public void requestPermission (){
		if (permissionRequested == "") requestPermission (searchDevice());
	}

	private UsbDevice searchDevice () {
		if (Build.VERSION.SDK_INT <= 11) return null;
		Log.d(TAG, "CameraManager.searchDevice started");
		UsbDevice device = null;
		UsbManager mUsbManager = (UsbManager)appContext.getSystemService(Context.USB_SERVICE);
		try{
			for (UsbDevice lDevice :  mUsbManager.getDeviceList().values()) {
				UsbLogger.log (TAG, "  searchDevice: " +lDevice.getDeviceName() +", class: " +lDevice.getDeviceClass() +", protocol: " +lDevice.getDeviceProtocol());
				if (lDevice.getDeviceProtocol() == 0 && device == null) device = lDevice;	// select first matching device  
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return device;
	}

	private void requestPermission (UsbDevice device) {
		if (device != null) {
			if (bi != null && bi.device != null && bi.device.getDeviceName().equals(device.getDeviceName())){
				UsbLogger.log (TAG, "requestPermission / already connected: " + device.getDeviceName());
				return;
			}
			permissionRequested = device.getDeviceName();
			UsbManager mUsbManager = (UsbManager)appContext.getSystemService(Context.USB_SERVICE);
			mUsbManager.requestPermission(device, mPermissionIntent);
			UsbLogger.log (TAG, "requestPermission: " + device.getDeviceName());
		}
	}
	
	private void sendMessage (int msgWhat) {
		if (handler == null) return;  // Do nothing without handler
		Message msg = new Message ();
		msg.what = msgWhat;
		if (bi != null && bi.device != null) msg.obj = bi.device.getDeviceName();
		handler.sendMessage(msg);
	}
	
	public void initDevice (UsbDevice device) {
		if (device != null){
			Log.d(TAG, "### initDevice: "+device.getDeviceName());
//			synchronized (this) 
			{
				if ((bi!= null && bi.device != null && device.getDeviceName().equals(bi.device.getDeviceName())
						)) {
					UsbLogger.log (TAG, "initDevice: already connected: " +bi.device.getDeviceName());
					sendMessage (MSG_CONNECTED);
					return;
				}
				try {	
					UsbManager mUsbManager = (UsbManager)appContext.getSystemService(Context.USB_SERVICE);
//					UsbDeviceConnection mUsbDeviceConnection = mUsbManager.openDevice(device);
//					Log.d(TAG, "initDevice: mUsbDeviceConnection " +(mUsbDeviceConnection != null));
					bi = new BaselineInitiator (device, mUsbManager.openDevice(device));
					UsbLogger.log (TAG, "initDevice: " +(bi != null ? bi.device.getVendorId() : " bi=null"));
					// Select appropriate deviceInitiator, VIDs see http://www.linux-usb.org/usb.ids
					if (bi.device.getVendorId() == EosInitiator.CANON_VID) {
						try {
							bi.getClearStatus();
							bi.close();
						} catch (PTPException e) {e.printStackTrace();}
						Log.d(TAG, "Device is CANON, open EOSInitiator");
						bi = new EosInitiator (device, mUsbManager.openDevice(device));
					}
					else if (device.getVendorId() == NikonInitiator.NIKON_VID) {
						try {
							bi.getClearStatus();
							bi.close();
						} catch (PTPException e) {e.printStackTrace();}
						Log.d(TAG, "Device is Nikon, open NikonInitiator");
						bi = new NikonInitiator (device, mUsbManager.openDevice(device));
					}
					bi.openSession();	
					
					permissionRequested = device.getDeviceName();  // to avoid double permission requests
					sendMessage (MSG_CONNECTED);
					UsbLogger.log  (TAG, "Device " +bi.info.model +" connected, session opened");
				} catch (PTPException e) {
					e.printStackTrace();
					detachDevice ();
				}
			}
		}
	}

	/**
	 *  call detachDevice () in onDestroy!
	 */
	public void detachDevice () {
		permissionRequested = "";
		sendMessage (MSG_DISCONNECTED);
		if (bi != null /*&& mDevice.equals(device)*/) {			
			if (bi.device != null) Log.d(TAG, "detachDevice: " +bi.device.getDeviceName());
			try {
				EosInitiator.eventListenerRunning = false;
				try {
					bi.getClearStatus();
				} catch (Exception e) {
					e.printStackTrace();
				}
				bi.close();
				handler = null;
			} catch (PTPException e) {
				e.printStackTrace();
				bi.device = null;
			}
		} 
		try {
			appContext.unregisterReceiver(mUsbReceiver);
		} catch (Exception e) { e.printStackTrace();}
	}

	// receive Broadcasts
	BroadcastReceiver mUsbReceiver = new BroadcastReceiver() {
		public void onReceive(Context context, Intent intent) {
			String action = intent.getAction();
			UsbLogger.log (TAG, "Receiver  onReceive: " +action);
			UsbDevice device = (UsbDevice)intent.getParcelableExtra(UsbManager.EXTRA_DEVICE);
			if (UsbManager.ACTION_USB_DEVICE_ATTACHED.equals(action) /*|| (device != null)*/) {
				UsbLogger.log (TAG, "USB_DEVICE_ATTACHED, init: " +device.getDeviceName());
				initDevice (device);
			} else if (UsbManager.ACTION_USB_DEVICE_DETACHED.equals(action)) {
				UsbLogger.log (TAG, "USB_DEVICE_DETACHED");
				if (bi != null && bi.session != null) bi.session.close();
				detachDevice ();
			} else if (ACTION_USB_PERMISSION.equals(action)) {
	            synchronized (this) {
	                if (intent.getBooleanExtra(UsbManager.EXTRA_PERMISSION_GRANTED, false)) {
	                	UsbLogger.log (TAG, "mUsbReceiver  EXTRA_PERMISSION_GRANTED, init: " +device.getDeviceName());
	                    if(device != null){
	        				initDevice (device);
	                   }
	                } 
	                else {
	                	UsbLogger.log (TAG, "permission denied for device " + device);
	                }
	            }
	        }	    			        
		}
	};

	// Converts String into numeric value, accepts also  fractional expressions like 1/20 (returns 0.05)
    @SuppressWarnings("finally")
	public static double getNumeric (String s) {
    	double r = 0;
    	if (s.contains("/")) {  // resolve fractional expressions like 1/20 (returns 0.05)
    		double p1 = getNumeric (s.substring(0, s.indexOf("/")));
    		double p2 = getNumeric (s.substring(s.indexOf("/")+1));
    		if (p2 != 0) {return p1/p2;} else return p1;
    	}

    	s = s.replaceAll( "[^\\d.,+-]", " " ).trim();
    	if (s.contains(" ")) s = s.substring(0, s.indexOf(" "));
    	s = s.replace(",", ".");      // avoids problems with decimal comma (e.g. German language)
    	s = s.replace("+", "");
    	try {
    		r = Double.valueOf (s);
    	return r;
    	}
    	finally {return r;}
    }
	
}
