/* Copyright 2010 by Stefano Fornari
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package com.android.ptplib.eos;

import java.util.ArrayList;
import java.util.List;



/**
 *
 * @author ste
 */
public class EosEvent implements EosEventConstants {

    /**
     * Event code
     */
    private int code;

    /**
     * Params
     */
    private List params;

    /**
     * Creates a new parser with the given data
     *
     * @param data the events data - NOT NULL
     */
    public EosEvent() {
        params = new ArrayList();
    }

    public void setCode(int code) {
        this.code = code;
    }

    public int getCode() {
        return code;
    }

    /**
     * @param i the parameter index
     * @param param the param to set
     */
    public void setParam(int i, Object value) {
        if (i<0) {
            throw new IllegalArgumentException("param index cannot be < 0");
        }
        if (params.size() <= i) {
            ArrayList newParams = new ArrayList(i);
            newParams.addAll(params);
            params = newParams;
            for (int j=params.size(); j<i; ++j) {
                params.add(null);
            }
        }
        params.set(i-1, value);
    }

    public void setParam(int i, int value) {
        setParam(i, new Integer(value));
    }

    public Object getParam(int i) {
        if ((i < 1) || (i > getParamCount())) {
            throw new IllegalArgumentException(
                "index " + i + " out of range (0-" + getParamCount() + ")"
            );
        }
        return params.get(i-1);
    }

    public int getIntParam(int i) {
        return ((Integer)getParam(i)).intValue();
    }

    public String getStringParam(int i) {
        return (String)getParam(i);
    }

    /**
     *
     * @return the number of parameters in this event
     */
    public int getParamCount() {
        return params.size();
    }

    public static String getEventName (int code){
    	switch (code) {
    	case  EosEventRequestGetEvent         : return "EosEventRequestGetEvent";
    	case  EosEventObjectAddedEx           : return "EosEventObjectAddedEx";
    	case  EosEventObjectRemoved           : return "EosEventObjectRemoved";
    	case  EosEventRequestGetObjectInfoEx  : return "EosEventRequestGetObjectInfoEx";
    	case  EosEventStorageStatusChanged    : return "EosEventStorageStatusChanged";
    	case  EosEventStorageInfoChanged      : return "EosEventStorageInfoChanged";
    	case  EosEventRequestObjectTransfer   : return "EosEventRequestObjectTransfer";
    	case  EosEventObjectInfoChangedEx     : return "EosEventObjectInfoChangedEx";
    	case  EosEventObjectContentChanged    : return "EosEventObjectContentChanged";
    	case  EosEventPropValueChanged        : return "EosEventPropValueChanged";
    	case  EosEventAvailListChanged        : return "EosEventAvailListChanged";
    	case  EosEventCameraStatusChanged     : return "EosEventCameraStatusChanged";
    	case  EosEventWillSoonShutdown        : return "EosEventWillSoonShutdown";
    	case  EosEventShutdownTimerUpdated    : return "EosEventShutdownTimerUpdated";
    	case  EosEventRequestCancelTransfer   : return "EosEventRequestCancelTransfer";
    	case  EosEventRequestObjectTransferDT : return "EosEventRequestObjectTransferDT";
    	case  EosEventRequestCancelTransferDT : return "EosEventRequestCancelTransferDT";
    	case  EosEventStoreAdded              : return "EosEventStoreAdded";
    	case  EosEventStoreRemoved            : return "EosEventStoreRemoved";
    	case  EosEventBulbExposureTime        : return "EosEventBulbExposureTime";
    	case  EosEventRecordingTime           : return "EosEventRecordingTime";
    	case  EosEventAfResult                : return "EosEventRequestObjectTransferTS";
    	case  EosEventRequestObjectTransferTS : return "EosEventAfResult";
    	}
	return "0x" +Integer.toHexString(code);
    }

    public static String getEventPropertyName (int property){
    	switch (property) {
    	case 0xD101: return "EosPropAperture";
    	case 0xD102: return "EosPropShutterSpeed";
    	case 0xD103: return "EosPropISOSpeed";
    	case 0xD104: return "EosPropExpCompensation";
    	case 0xD105: return "EosPropAutoExposureMode";
    	case 0xD106: return "EosPropDriveMode";
    	case 0xD107: return "EosPropMeteringMode";
    	case 0xD108: return "EosPropFocusMode";
    	case 0xD109: return "EosPropWhiteBalance";
    	case 0xD10A: return "EosPropColorTemperature";
    	case 0xD10B: return "EosPropWhiteBalanceAdjustA";
    	case 0xD10C: return "EosPropWhiteBalanceAdjustB";
    	case 0xD10D: return "EosPropWhiteBalanceXA";
    	case 0xD10E: return "EosPropWhiteBalanceXB";
    	case 0xD10F: return "EosPropColorSpace";
    	case 0xD110: return "EosPropPictureStyle";
    	case 0xD111: return "EosPropBatteryPower";
    	case 0xD112: return "EosPropBatterySelect";
    	case 0xD113: return "EosPropCameraTime";
    	case 0xD115: return "EosPropOwner";
    	case 0xD116: return "EosPropModelID";
    	case 0xD119: return "EosPropPTPExtensionVersion";
    	case 0xD11A: return "EosPropDPOFVersion";
    	case 0xD11B: return "EosPropAvailableShots";
    	case 0xD11C: return "EosPropCaptureDestination";
    	case 0xD11D: return "EosPropBracketMode";
    	case 0xD11E: return "EosPropCurrentStorage";
    	case 0xD11F: return "EosPropCurrentFolder";
    	case 0xD120: return "EosPropImageFormat";
    	case 0xD121: return "EosPropImageFormatCF";
    	case 0xD122: return "EosPropImageFormatSD";
    	case 0xD123: return "EosPropImageFormatExtHD";
    	case 0xD130: return "EosPropCompressionS";
    	case 0xD131: return "EosPropCompressionM1";
    	case 0xD132: return "EosPropCompressionM2";
    	case 0xD133: return "EosPropCompressionL";
    	case 0xD140: return "EosPropPCWhiteBalance1";
    	case 0xD141: return "EosPropPCWhiteBalance2";
    	case 0xD142: return "EosPropPCWhiteBalance3";
    	case 0xD143: return "EosPropPCWhiteBalance4";
    	case 0xD144: return "EosPropPCWhiteBalance5";
    	case 0xD145: return "EosPropMWhiteBalance";
    	case 0xD150: return "EosPropPictureStyleStandard";
    	case 0xD151: return "EosPropPictureStylePortrait";
    	case 0xD152: return "EosPropPictureStyleLandscape";
    	case 0xD153: return "EosPropPictureStyleNeutral";
    	case 0xD154: return "EosPropPictureStyleFaithful";
    	case 0xD155: return "EosPropPictureStyleMonochrome";
    	case 0xD160: return "EosPropPictureStyleUserSet1";
    	case 0xD161: return "EosPropPictureStyleUserSet2";
    	case 0xD162: return "EosPropPictureStyleUserSet3";
    	case 0xD170: return "EosPropPictureStyleParam1";
    	case 0xD171: return "EosPropPictureStyleParam2";
    	case 0xD172: return "EosPropPictureStyleParam3";
    	case 0xD17F: return "EosPropFlavorLUTParams";
    	case 0xD180: return "EosPropCustomFunc1";
    	case 0xD181: return "EosPropCustomFunc2";
    	case 0xD182: return "EosPropCustomFunc3";
    	case 0xD183: return "EosPropCustomFunc4";
    	case 0xD184: return "EosPropCustomFunc5";
    	case 0xD185: return "EosPropCustomFunc6";
    	case 0xD186: return "EosPropCustomFunc7";
    	case 0xD187: return "EosPropCustomFunc8";
    	case 0xD188: return "EosPropCustomFunc9";
    	case 0xD189: return "EosPropCustomFunc10";
    	case 0xD18A: return "EosPropCustomFunc11";
    	case 0xD18B: return "EosPropCustomFunc12";
    	case 0xD18C: return "EosPropCustomFunc13";
    	case 0xD18D: return "EosPropCustomFunc14";
    	case 0xD18E: return "EosPropCustomFunc15";
    	case 0xD18F: return "EosPropCustomFunc16";
    	case 0xD190: return "EosPropCustomFunc17";
    	case 0xD191: return "EosPropCustomFunc18";
    	case 0xD192: return "EosPropCustomFunc19";
    	case 0xD1A0: return "EosPropCustomFuncEx";
    	case 0xD1A1: return "EosPropMyMenu";
    	case 0xD1A2: return "EosPropMyMenuList";
    	case 0xD1A3: return "EosPropWftStatus";
    	case 0xD1A4: return "EosPropWftInputTransmission";
    	case 0xD1A5: return "EosPropHDDirectoryStructure";
    	case 0xD1A6: return "EosPropBatteryInfo";
    	case 0xD1A7: return "EosPropAdapterInfo";
    	case 0xD1A8: return "EosPropLensStatus";
    	case 0xD1A9: return "EosPropQuickReviewTime";
    	case 0xD1AA: return "EosPropCardExtension";
    	case 0xD1AB: return "EosPropTempStatus";
    	case 0xD1AC: return "EosPropShutterCounter";
    	case 0xD1AD: return "EosPropSpecialOption";
    	case 0xD1AE: return "EosPropPhotoStudioMode";
    	case 0xD1AF: return "EosPropSerialNumber";
    	case 0xD1B0: return "EosPropEVFOutputDevice";
    	case 0xD1B1: return "EosPropEVFMode";
    	case 0xD1B2: return "EosPropDepthOfFieldPreview";
    	case 0xD1B3: return "EosPropEVFSharpness";
    	case 0xD1B4: return "EosPropEVFWBMode";
    	case 0xD1B5: return "EosPropEVFClickWBCoeffs";
    	case 0xD1B6: return "EosPropEVFColorTemp";
    	case 0xD1B7: return "EosPropExposureSimMode";
    	case 0xD1B8: return "EosPropEVFRecordStatus";
    	case 0xD1BA: return "EosPropLvAfSystem";
    	case 0xD1BB: return "EosPropMovSize";
    	case 0xD1BC: return "EosPropLvViewTypeSelect";
    	case 0xD1D0: return "EosPropArtist";
    	case 0xD1D1: return "EosPropCopyright";
    	case 0xD1D2: return "EosPropBracketValue";
    	case 0xD1D3: return "EosPropFocusInfoEx";
    	case 0xD1D4: return "EosPropDepthOfField";
    	case 0xD1D5: return "EosPropBrightness";
    	case 0xD1D6: return "EosPropLensAdjustParams";
    	case 0xD1D7: return "EosPropEFComp";
    	case 0xD1D8: return "EosPropLensName";
    	case 0xD1D9: return "EosPropAEB";
    	case 0xD1DA: return "EosPropStroboSetting";
    	case 0xD1DB: return "EosPropStroboWirelessSetting";
    	case 0xD1DC: return "EosPropStroboFiring";
    	case 0xD1DD: return "EosPropLensID";
    	}
    	return "0x" +Integer.toHexString(property);
    }

}
